/* $Workfile: OtxApi.c $   $Revision: 13 $    $Modtime: 2/27/01 3:21p $  */
/*--------------------------------------------------------------------------*\

  Odin TeleSystems Inc.

  SAA-1007-1 OTX DSP C54x Software Development Kit (SDK)

  Implementation of the Standard API functions that should not be
  modified by the user.

\*--------------------------------------------------------------------------*/

#include <string.h>  /* for memset() */

#include "DspIoctl.h"
#include "OtxRev.h"

#include "OtxApi.h"

/*--------------------------------------------------------------------------*\
  Macros
\*--------------------------------------------------------------------------*/
/* @type C54X_SET_INT_MASK_BIT | Macro for Masking Interrupts */
#define C54X_SET_INT_MASK_BIT   asm("\t ssbx    INTM")
/* @type C54X_CLR_INT_MASK_BIT | Macro for Unmasking Interrupts */
#define C54X_CLR_INT_MASK_BIT   asm("\t rsbx    INTM")


/*--------------------------------------------------------------------------*\
  Global Data
\*--------------------------------------------------------------------------*/
volatile unsigned short *g_h2dCmdAddrP      = (unsigned short *) OTXDSP_SDK_IOCTL_CODE_ADDR;
volatile unsigned short *g_h2dHandleAddrP   = (unsigned short *) OTXDSP_SDK_IOCTL_HANDLE_ADDR;
volatile unsigned short *g_h2dTaskRefAddrP  = (unsigned short *) OTXDSP_SDK_IOCTL_TASKREF_ADDR;
volatile unsigned short *g_h2dDataLenAddrP  = (unsigned short *) OTXDSP_SDK_IOCTL_DATA_LEN_ADDR;
volatile unsigned short *g_h2dDataAddrP     = (unsigned short *) OTXDSP_SDK_IOCTL_DATA_ADDR;

static unsigned short *g_pCurrDspWriteAddr  = (unsigned short *) OTXDSP_SDK_EVENT_BUF_ADDR;

OtxDspHpiEventBufDiag g_hpiEventBufDiag;

OtxDspTimeStamp g_timeStamp;

volatile short g_bGotRxSerialPortInterrupt = OTXDSP_FALSE;
volatile short g_bGotTxSerialPortInterrupt = OTXDSP_FALSE;
volatile short g_bGotHostPortInterrupt     = OTXDSP_FALSE;

//short g_intmVal = 5;

/*--------------------------------------------------------------------------*\
  Certain Memory Mapped useful Registers
\*--------------------------------------------------------------------------*/
volatile unsigned short * const C54X_IMR_REG    = (volatile unsigned short *) 0x0000;
volatile unsigned short * const C54X_IFR_REG    = (volatile unsigned short *) 0x0001;
volatile unsigned short * const C54X_ST0_REG    = (volatile unsigned short *) 0x0006;
volatile unsigned short * const C54X_ST1_REG    = (volatile unsigned short *) 0x0007;
volatile unsigned short * const C54X_PMST_REG   = (volatile unsigned short *) 0x001D;
volatile unsigned short * const C54X_BDRR_REG   = (volatile unsigned short *) 0x0020;
volatile unsigned short * const C54X_BDXR_REG   = (volatile unsigned short *) 0x0021;
volatile unsigned short * const C54X_SPC0_REG   = (volatile unsigned short *) 0x0022;
volatile unsigned short * const C54X_TIM_REG    = (volatile unsigned short *) 0x0024;
volatile unsigned short * const C54X_PRD_REG    = (volatile unsigned short *) 0x0025;
volatile unsigned short * const C54X_TCR_REG    = (volatile unsigned short *) 0x0026;
volatile unsigned short * const C54X_TRCV_REG   = (volatile unsigned short *) 0x0030;
volatile unsigned short * const C54X_TDXR_REG   = (volatile unsigned short *) 0x0031;
volatile unsigned short * const C54X_SPC1_REG   = (volatile unsigned short *) 0x0032;
volatile unsigned short * const C54X_SWWSR_REG  = (volatile unsigned short *) 0x0028;
volatile unsigned short * const C54X_BSCR_REG   = (volatile unsigned short *) 0x0029;
volatile unsigned short * const C54X_HPIC_REG   = (volatile unsigned short *) 0x002C;
volatile unsigned short * const C54X_BSPCE_REG  = (volatile unsigned short *) 0x0023;
volatile unsigned short * const C54X_AXR_REG    = (volatile unsigned short *) 0x0038;
volatile unsigned short * const C54X_BKX_REG    = (volatile unsigned short *) 0x0039;
volatile unsigned short * const C54X_ARR_REG    = (volatile unsigned short *) 0x003A;
volatile unsigned short * const C54X_BKR_REG    = (volatile unsigned short *) 0x003B;
volatile unsigned short * const C54X_CLKMD_REG  = (volatile unsigned short *) 0x0058;

/*--------------------------------------------------------------------------*\
  Interrupt Service Routines
\*--------------------------------------------------------------------------*/

interrupt void c_int19(
/*--------------------------------------------------------------------------*\

  SYNOPSIS

    TIMER INTERRUPT SERVICE ROUTINE

    This interrupt service routine will be executed every time TIM 
    decrements to 0.

    The timer interrupt is used to create an accurate timestamp 
    (64 bits) for notify events to be sent to the host. The timer interrupt
    frequency is 10.000MHz for ASM and PCI boards and 10.036MHz for PCMCIA boards.

\*--------------------------------------------------------------------------*/
  void
)
{

  if (g_timeStamp.nLowPart == 0xFFFF) {
    if (g_timeStamp.nMidPart == 0xFFFF) {
      g_timeStamp.nHighPart++;
    }
    g_timeStamp.nMidPart++;
  }
  g_timeStamp.nLowPart++;

}


interrupt void c_int20(
/*--------------------------------------------------------------------------*\
  @func
    Serial Port Receive Interrupt Service Routine

    The DSP is connected to the system highway (32 timeslots).

    We will get this interrupt every 25 highway frames (since the
    autobuffer is 800 words long, which is 50 frames (50*16=800), and we
    get this interrupt when the autobuffer is half full and also when is
    completely full).

\*--------------------------------------------------------------------------*/
  void
)
{
  *C54X_BSPCE_REG |= 0x2000;   // Set BRE=1 to enable autobuffering again (since we are running with HALTR=1)

  g_bGotRxSerialPortInterrupt = OTXDSP_TRUE;
}



interrupt void c_int21(
/*--------------------------------------------------------------------------*\
  @func
    Serial Port Transmit Interrupt Service Routine
\*--------------------------------------------------------------------------*/
  void
)
{
  *C54X_BSPCE_REG |= 0x0400;   // Set BXE=1 to enable autobuffering again (since we are running with HALTX=1)

  g_bGotTxSerialPortInterrupt = OTXDSP_TRUE;
}




interrupt void c_int22(
/*--------------------------------------------------------------------------*\
  @func
\*--------------------------------------------------------------------------*/
  void
)
{
  // Nothing
}


interrupt void c_int23(
/*--------------------------------------------------------------------------*\
  @func
\*--------------------------------------------------------------------------*/
  void
)
{
 // Nothing
}



interrupt void c_int25(
/*--------------------------------------------------------------------------*\
  @func
    Host Port Interrupt. Activated when the host wants our attention.
\*--------------------------------------------------------------------------*/
  void
)
{
  g_bGotHostPortInterrupt = OTXDSP_TRUE;
}



/*--------------------------------------------------------------------------*\
  API Functions
\*--------------------------------------------------------------------------*/

void OtxDspInit(
/*--------------------------------------------------------------------------*\
    Run Standard OTX DSP SDK API Initializations
\*--------------------------------------------------------------------------*/
  void
)
{
  // Configure the Processor
  OtxDspInitProcessor();

  // Initialize Standard Global Data
  OtxDspInitGlobalData();

  // Initialize the Host Port Interface
  OtxDspInitHostPortInterface();

  // Initialize the Buffered Serial Ports
  OtxDspInitBufferedSerialPorts();

  // Initialize Interrupts. Mask unwanted interrupts
  OtxDspInitInterrupts(0x0200);  // 0x0200=allow Host Port Interrupt

}




void OtxDspInitProcessor(
/*--------------------------------------------------------------------------*\
    Sets up the processor to the desired operating mode.
\*--------------------------------------------------------------------------*/
  void
)
{
  *C54X_ST1_REG = 0x6900;    // 0x6900 is the value of ST1 when Code Composer starts
  *C54X_PMST_REG |= 0x0002;  // Turn off CLKOUT 

  *C54X_PMST_REG &= ~0xFF80; // Clear Interrupt vector pointer
  *C54X_PMST_REG |=  0x0180; // Set Interrupt vector pointer to 0x180 (as specified in MAIN.CMD)

  *C54X_SWWSR_REG = 0        // Set Software Wait States
  | (0 << 12) // Wait states for I/O space 0x0000-0xFFFF (not connected)
  | (1 << 9)  // Wait states for data space 0x8000-0xFFFF (external memory, if available)
  | (0 << 6)  // Wait states for data space 0x0000-0x7FFF (internal memory)
  | (1 << 3)  // Wait states for program space 0x8000-0xFFFF (external memory, if available)
  | (0 << 0)  // Wait states for program space 0x0000-0x7FFF (internal memory)
  ;

  *C54X_CLKMD_REG = 0x5320;  // Set CLK multiplier to 6 (CLKINmax = 13.333MHz), with a 160 lockup time
  *C54X_CLKMD_REG = 0x5322;  // Start lockup sequence
  while (!(*C54X_CLKMD_REG & 0x0001)); // Wait for lockup sequence to complete

}


void OtxDspInitGlobalData(
/*--------------------------------------------------------------------------*\
    Initalizes the global data used by the Standard API functions.
\*--------------------------------------------------------------------------*/
  void
)
{
  g_pCurrDspWriteAddr  = (unsigned short *) OTXDSP_SDK_EVENT_BUF_ADDR;

  g_hpiEventBufDiag.cwFreeBuffer = g_hpiEventBufDiag.cwMinFreeBuffer = OTXDSP_SDK_EVENT_BUF_LEN;
  g_hpiEventBufDiag.coLostEvents = 0;  

  memset(&g_timeStamp, 0, sizeof(OtxDspTimeStamp));

}


void OtxDspInitInterrupts(
/*--------------------------------------------------------------------------*\
    Initialize and unmask specified interrupts.
\*--------------------------------------------------------------------------*/
  unsigned short valueIMR       /* @parm Interrupts to be unmasked */
)
{
  C54X_SET_INT_MASK_BIT;   // Globally disable interrupts by setting the INTM bit 

  *C54X_IFR_REG = valueIMR;          /* Clear any pending interrupts for this interrupt(s) */

  *C54X_IMR_REG |= valueIMR;         /* Allow this interrupt(s) */

  C54X_CLR_INT_MASK_BIT;  // Globally enable interrupts by clearing the INTM bit
}


void OtxDspInitHostPortInterface(
/*--------------------------------------------------------------------------*\
    Initialize the Host Port Interface
\*--------------------------------------------------------------------------*/
  void
)
{
  asm("\t STM    #02h, hpic");  /* Switch to SAM mode (if we are not already in that mode) */
  C54X_NOP;                          /* This can't be IDLE2/3 instruction. See Chapter 8.5.5 in the CPU and Peripherals Reference Book) */
  C54X_NOP;                          /* A few extra can't hurt */
  C54X_NOP;                          /* A few extra can't hurt */
}


void OtxDspInitBufferedSerialPorts(
/*--------------------------------------------------------------------------*\
    Initialize the Buffered Serial Ports (BSP)
\*--------------------------------------------------------------------------*/
  void
)
{
  *C54X_SPC0_REG = 0x0000;      /* Reset BSP */

  *C54X_SPC0_REG = 0x0000;      /* reset SP0                            */
                                /*     =0 (b0) : reserved, always zero  */
                                /* DBL =0 (b1) : loopback mode disabled */
                                /* FO  =0 (b2) : 16-bit word operation  */
                                /* FSM =1 (b3) : frame sync control     */
                                /* MCM =0 (b4) : external CLKX          */
                                /* TXM =0 (b5) : external FSK           */
                                /* XRST=0 (b6) : Xmit reset             */
                                /* RRST=0 (b7) : Recv reset             */

  OtxDspInitInterrupts(0x0030);  /* Clear pending TX and RX interrupts and enable them */


  *C54X_BSPCE_REG = 0xB400;     /*     BSP Extended functions register */
                                /*  0  CLKDV=0  (b0-4) : internal transmit clock division factor */
                                /*  0  FSP  =0  (b5) : frame sync polarity */
                                /*  0  CLKP =0  (b6) : clock polarity */
                                /*  0  FE   =0  (b7) : format extension */
                                /*  1  FIG  =0  (b8) : frame ignore */
                                /*  0  PCM  =0  (b9) : PCM mode */
                                /*  0  BSP Autobuffering Unit bits */
                                /*  1  BXE  =1 (b10) : autobuffering transmit enable */
                                /*  0  XH   =0 (b11) : transmit buffer half transmitted */
                                /*  1  HALTX=0 (b12) : autobuffering transmit halt */
                                /*  1  BRE  =1 (b13) : autobuffering receive enable */
                                /*  0  RH   =0 (b14) : receive buffer half transmitted */
                                /*  1  HALTR=0 (b15) : autobuffering receive halt */

  *C54X_BKX_REG = OTXDSP_BSP_TXBUF_SIZE;          /* Size of transmit buffer */
  *C54X_AXR_REG = OTXDSP_BSP_TXBUF_ADDR - 0x800;  /* Relative start address of transmit buffer (0x0800 + AXR) */
  *C54X_BKR_REG = OTXDSP_BSP_RXBUF_SIZE;          /* Size of receive buffer */
  *C54X_ARR_REG = OTXDSP_BSP_RXBUF_ADDR - 0x800;  /* Relative start address of receive buffer (0x0800 + ARR) */

  *C54X_SPC0_REG = 0x00C0;      /* take SP0 out of reset                */
                                /* XRST=0 (b6) : Xmit reset             */
                                /* RRST=0 (b7) : Recv reset             */
}



/*--------------------------------------------------------------------------*\
  @func
\*--------------------------------------------------------------------------*/
short OtxDspStartProgram(
  OtxDspSdkIoInStartProgramS *pSin
)
{

  // Check that the driver revision matches the revision of the DSP SDK
  if (pSin->nDrvRevNoMajor != OTX_DRV_REV_NO_MAJOR) {
    return OTXDSP_SDK_E_UNSUPPORTED_VERSION;
  }
  if (pSin->nDrvRevNoMinor != OTX_DRV_REV_NO_MINOR) {
    return OTXDSP_SDK_E_UNSUPPORTED_VERSION;
  }
  if (pSin->nDrvRevNoPr != OTX_DRV_REV_NO_PR) {
    return OTXDSP_SDK_E_UNSUPPORTED_VERSION;
  }
  if (pSin->nDrvRevNoPr == 0) {
    if (pSin->nDrvRevNoPversion != OTX_DRV_REV_NO_PVERSION) {
      return OTXDSP_SDK_E_UNSUPPORTED_VERSION;
    }
  }

  // Check if we are in DIV mode or PLL mode
  if (*C54X_CLKMD_REG & 0x0001) {
    // Since we are already in PLL mode (we are just changing the multiplier), we
    // must first switch over to DIV mode before we switch back to the new PLL
    // mode.
    *C54X_CLKMD_REG = 0;  // switch to DIV mode
    while (*C54X_CLKMD_REG & 0x0001); // Wait until the DSP has switched over to DIV mode
    *C54X_CLKMD_REG = 0;  // Reset PLLON/OFF now when status is in DIV mode
  }

  *C54X_CLKMD_REG = pSin->nClkMdVal; // Reconfigure PLLMULL, PLLCOUNT, and PLLDIV
  *C54X_CLKMD_REG = pSin->nClkMdVal | 0x0002; // Set PLLNDIV high to start lockup sequence
  while (!(*C54X_CLKMD_REG & 0x0001)); // Wait for lockup sequence to complete

  return OTXDSP_SDK_S_OK;

}

/*--------------------------------------------------------------------------*\
  @func
\*--------------------------------------------------------------------------*/
short OtxDspInitTimer(
  void
)
{
  C54X_SET_INT_MASK_BIT;   // Globally disable interrupts by setting the INTM bit 

  *C54X_TCR_REG = 0x0010;    // Turn timer off 

  *C54X_TIM_REG = 0xFFFF;    // Timer counts down so put maximum value in the timer

  memset(&g_timeStamp, 0, sizeof(OtxDspTimeStamp));

  OtxDspInitInterrupts(0x0008);     // Clear pending TINT interrupts and enable them (and enable global interrupts)

  *C54X_TCR_REG = 0x0007;    // turn timer on; Divide-down-ratio=8 => For a 80MHz CLKOUT [T=12.5ns] => 8*12.5ns = 100ns "ticks"

  return OTXDSP_SDK_S_OK;

}



void OtxDspCommandCompleteData(
/*--------------------------------------------------------------------------*\
    Completes an IO Control call from the host. Passes a 
    arbitrary size of return data from a IO Control call back to the host.
\*--------------------------------------------------------------------------*/
  unsigned short hHandle, 
  unsigned short *pData,
  unsigned short dataLen
) 
{ 
  unsigned short i;
  unsigned short *pTempDataAddr = (unsigned short *)g_h2dDataAddrP;

  *g_h2dHandleAddrP  = hHandle;
  *g_h2dDataLenAddrP = dataLen;
  for (i = 0; i < dataLen; i++) {
    *(pTempDataAddr) = *pData++; 
    pTempDataAddr++;
  }
  *g_h2dCmdAddrP = OTXDSP_SDK_IO_NO_COMMAND; // Command completed 

  *C54X_HPIC_REG = C54X_HINTMASK; /* Assert HINT# towards the host */
  C54X_NOP;                  /* Let it sink in... */
  C54X_NOP;                  /* Let it sink in... */
  C54X_NOP;                  /* Let it sink in... */
}

void OtxDspCommandCompleteRc(
/*--------------------------------------------------------------------------*\
    Completes an IO Control call from the host. Passes a 
    return code from a IO Control call back to the host.
\*--------------------------------------------------------------------------*/
  unsigned short hHandle,
  unsigned short nResult
)
{
  *g_h2dHandleAddrP  = hHandle;
  *g_h2dDataLenAddrP = sizeof(unsigned short);
  *g_h2dDataAddrP = nResult;
  *g_h2dCmdAddrP = OTXDSP_SDK_IO_NO_COMMAND; // Command completed

  *C54X_HPIC_REG = C54X_HINTMASK; /* Assert HINT# towards the host */
  C54X_NOP;                  /* Let it sink in... */
  C54X_NOP;                  /* Let it sink in... */
  C54X_NOP;                  /* Let it sink in... */
}


void OtxDspCommandComplete(
/*--------------------------------------------------------------------------*\
    Completes an IO Control call from the host. No data is passed back
    to the host.
\*--------------------------------------------------------------------------*/
  unsigned short hHandle
)
{
  *g_h2dHandleAddrP  = hHandle;
  *g_h2dDataLenAddrP = 0;
  *g_h2dCmdAddrP = OTXDSP_SDK_IO_NO_COMMAND; // Command completed 

  *C54X_HPIC_REG = C54X_HINTMASK; /* Assert HINT# towards the host */
  C54X_NOP;                  /* Let it sink in... */
  C54X_NOP;                  /* Let it sink in... */
  C54X_NOP;                  /* Let it sink in... */
}


short OtxDspSetHostEvent(
/*--------------------------------------------------------------------------*\
    Generates a driver event for the host
\*--------------------------------------------------------------------------*/
  unsigned short nEvent,         /* @parm Event to be send to the host */
  unsigned short hHandle,        /* @parm Handle to the user logical device sending the event */
  unsigned short nTaskRef,       /* @parm Associated Task Reference */
  unsigned short *pData,         /* @parm Pointer to the data to be sent to the host */
  unsigned short nDataLen        /* @parm Length of the data to be sent */
)
{
  unsigned short i;
  volatile unsigned short *pEventAddr;
  volatile unsigned short nCurrHostReadAddr = *((unsigned short *) OTXDSP_SDK_EVENT_BUF_PROCESSING_ADDR);


  if ((unsigned short)g_pCurrDspWriteAddr >= nCurrHostReadAddr) {
    g_hpiEventBufDiag.cwFreeBuffer = OTXDSP_SDK_EVENT_BUF_LEN - ((unsigned short)g_pCurrDspWriteAddr - nCurrHostReadAddr);
  } else {
    g_hpiEventBufDiag.cwFreeBuffer = nCurrHostReadAddr - (unsigned short)g_pCurrDspWriteAddr;
  }

  if (g_hpiEventBufDiag.cwFreeBuffer < g_hpiEventBufDiag.cwMinFreeBuffer) {
    g_hpiEventBufDiag.cwMinFreeBuffer = g_hpiEventBufDiag.cwFreeBuffer;
  }
  
  // If we have enough free buffer space then insert the command in the buffer
  if (g_hpiEventBufDiag.cwFreeBuffer >= (nDataLen + 8)) {   // +8 for the nEvent+handle+taskref+timestamp*4+NO_CMD
    // Don't write the event notify code yet. 
    // Just save the address where we want to write the code. 
    // We will write the event notify code when we have written the entire event packet 
    // to the buffer
    pEventAddr = g_pCurrDspWriteAddr; 

    g_pCurrDspWriteAddr++;
    if (g_pCurrDspWriteAddr >= (unsigned short *)(OTXDSP_SDK_EVENT_BUF_ADDR + OTXDSP_SDK_EVENT_BUF_LEN)) {
      g_pCurrDspWriteAddr = (unsigned short *)OTXDSP_SDK_EVENT_BUF_ADDR;
    }

#ifdef OTXDSP_SDK_EVENT_BUF_INCLUDE_HANDLE
    *g_pCurrDspWriteAddr = hHandle;

    g_pCurrDspWriteAddr++;
    if (g_pCurrDspWriteAddr >= (unsigned short *)(OTXDSP_SDK_EVENT_BUF_ADDR + OTXDSP_SDK_EVENT_BUF_LEN)) {
      g_pCurrDspWriteAddr = (unsigned short *)OTXDSP_SDK_EVENT_BUF_ADDR;
    }
#endif

#ifdef OTXDSP_SDK_EVENT_BUF_INCLUDE_TASKREF
    *g_pCurrDspWriteAddr = nTaskRef;

    g_pCurrDspWriteAddr++;
    if (g_pCurrDspWriteAddr >= (unsigned short *)(OTXDSP_SDK_EVENT_BUF_ADDR + OTXDSP_SDK_EVENT_BUF_LEN)) {
      g_pCurrDspWriteAddr = (unsigned short *)OTXDSP_SDK_EVENT_BUF_ADDR;
    }
#endif

#ifdef OTXDSP_SDK_EVENT_BUF_INCLUDE_TIMESTAMP
    // Time stamp
    {
      C54X_SET_INT_MASK_BIT;   // Globally disable interrupts by setting the INTM bit 

      *g_pCurrDspWriteAddr = g_timeStamp.nHighPart;
      
      g_pCurrDspWriteAddr++;
      if (g_pCurrDspWriteAddr >= (unsigned short *)(OTXDSP_SDK_EVENT_BUF_ADDR + OTXDSP_SDK_EVENT_BUF_LEN)) {
        g_pCurrDspWriteAddr = (unsigned short *)OTXDSP_SDK_EVENT_BUF_ADDR;
      }

      *g_pCurrDspWriteAddr = g_timeStamp.nMidPart;
      
      g_pCurrDspWriteAddr++;
      if (g_pCurrDspWriteAddr >= (unsigned short *)(OTXDSP_SDK_EVENT_BUF_ADDR + OTXDSP_SDK_EVENT_BUF_LEN)) {
        g_pCurrDspWriteAddr = (unsigned short *)OTXDSP_SDK_EVENT_BUF_ADDR;
      }

      *g_pCurrDspWriteAddr = g_timeStamp.nLowPart;
      
      g_pCurrDspWriteAddr++;
      if (g_pCurrDspWriteAddr >= (unsigned short *)(OTXDSP_SDK_EVENT_BUF_ADDR + OTXDSP_SDK_EVENT_BUF_LEN)) {
        g_pCurrDspWriteAddr = (unsigned short *)OTXDSP_SDK_EVENT_BUF_ADDR;
      }

      *g_pCurrDspWriteAddr = 0xFFFF - *C54X_TIM_REG;
      
      g_pCurrDspWriteAddr++;
      if (g_pCurrDspWriteAddr >= (unsigned short *)(OTXDSP_SDK_EVENT_BUF_ADDR + OTXDSP_SDK_EVENT_BUF_LEN)) {
        g_pCurrDspWriteAddr = (unsigned short *)OTXDSP_SDK_EVENT_BUF_ADDR;
      }

      C54X_CLR_INT_MASK_BIT;  // Globally enable interrupts by clearing the INTM bit
    }
#endif

    *(g_pCurrDspWriteAddr) = nDataLen; // data length

    g_pCurrDspWriteAddr++;
    if (g_pCurrDspWriteAddr >= (unsigned short *)(OTXDSP_SDK_EVENT_BUF_ADDR + OTXDSP_SDK_EVENT_BUF_LEN)) {
      g_pCurrDspWriteAddr = (unsigned short *)OTXDSP_SDK_EVENT_BUF_ADDR;
    }

    for (i = 0; i < nDataLen; i++) {
      *(g_pCurrDspWriteAddr) = *pData++;

      g_pCurrDspWriteAddr++;
      if (g_pCurrDspWriteAddr >= (unsigned short *)(OTXDSP_SDK_EVENT_BUF_ADDR + OTXDSP_SDK_EVENT_BUF_LEN)) {
        g_pCurrDspWriteAddr = (unsigned short *)OTXDSP_SDK_EVENT_BUF_ADDR;
      }
    }

    *(g_pCurrDspWriteAddr) = OTXDSP_SDK_EVENT_NO_COMMAND; // We put this here so that host knows when to stop reading from the command buffer

    // Now it is safe to write the event notify code (now when the entire packet is written to the buffer)
    *(pEventAddr) = nEvent; 

  } else {
    // Buffer overrun. The host is too slow. Record this error condition
    g_hpiEventBufDiag.coLostEvents++;
    return 0; // Error
  }


  *C54X_HPIC_REG = C54X_HINTMASK; /* Assert HINT# towards the host */
  C54X_NOP;                  /* Let it sink in... */
  C54X_NOP;                  /* Let it sink in... */
  C54X_NOP;                  /* Let it sink in... */

  return 1; // Success

}

